/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.04.11.
*   Chip         -  ATtiny13A
*
*   Tavvezerelt jelzolampa modell - piros/piros/feher
*
*                   +---------+
*                  ++ 1    8  |  +VCC
*   CONTROL - PB3  |  2    7  |  PB2 - L1
*   MIRROR  - PB4  |  3    6  |  PB1 - L2 (kozos, common)
*             GND  |  4    5  |  PB0 - L3
*                  +----------+
*
*   LED-ek vezerlese:  -  tukrozve:
*   piros1: L1-1, L2-0  -  L3-1, L2-0
*   piros2: L1-0, L2-1  -  L3-0, L2-1
*   feher:  L3-1, L2-0  -  L1-1, L2-0
*
*******************************************************************************/

//                        000...000
#define F_CPU               1200000                   // 1,2 Mhz




#include "tkiraaly_attiny13a.h"
#include <util/delay.h>




// L1
#define L1_OUTPUT           PB2_OUTPUT
#define L1_0                PB2_0
#define L1_1                PB2_1

// L2
#define L2_OUTPUT           PB1_OUTPUT
#define L2_0                PB1_0
#define L2_1                PB1_1

// L3
#define L3_OUTPUT           PB0_OUTPUT
#define L3_0                PB0_0
#define L3_1                PB0_1

// CONTROL
#define CONTROL_PULLUP      PB3_PULLUP
#define CONTROL_R           PB3_R

// MIRROR
#define MIRROR_PULLUP       PB4_PULLUP
#define MIRROR_R            PB4_R

// State:
#define RED1                L1_1; L2_0; L3_0
#define RED2                L1_0; L2_1; L3_1
#define WHITE               L1_0; L2_0; L3_1
#define BLANK               L1_0; L2_0; L3_0

#define MIRRORED_RED1       L3_1; L2_0; L1_0
#define MIRRORED_RED2       L3_0; L2_1; L1_1
#define MIRRORED_WHITE      L3_0; L2_0; L1_1




int main( void)
{
   U8 lamp_state= 0;                                  // lampa allapot szamlalo
   U8 pulse_lenght= 0;                                // impulzus hossz szamolasa
   U16 blink_counter= 0;                              // sarga villogas idozites szamlalo

   CONTROL_PULLUP;                                    // vezerles bemenete
   MIRROR_PULLUP;                                     // tukrozes jumper

   L1_OUTPUT;                                         // kimenetek
   L2_OUTPUT;
   L3_OUTPUT;

   for(;;)
   {
      if( MIRROR_R)                                   // lampa beallitasa MIRROR jumper szerint
      {
         switch( lamp_state )
         {

            case 0: RED1; break;                      // piros1
            case 1: RED2; break;                      // piros2
            case 3: WHITE; break;                     // feher
            case 4: BLANK;                            // sotet
         }
      }
      else
      {
         switch( lamp_state )
         {
            case 0: MIRRORED_RED1; break;             // piros1
            case 1: MIRRORED_RED2; break;             // piros2
            case 3: MIRRORED_WHITE; break;            // feher
            case 4: BLANK;                            // sotet
         }
      }     
      if( CONTROL_R == 0)                             // L - impulzus hossz szamolasa
      { 
         if( pulse_lenght < 9) pulse_lenght++;        // szamlalo ne fordulhasson korbe igen hosszu impulzus eseten
      }        
      else                                            // H - nincs impulzus
      {
         if( pulse_lenght)                            // volt impulzus, feldolgozas
         {
            if( pulse_lenght > 7) lamp_state= 0;      // 7 msec-nel hosszabb - piros
            else lamp_state= 3;                       // 7 msec-nel rovidebb - feher
            blink_counter= 0;
            pulse_lenght= 0;                          // kovetkezo impulzushoz
         }
      }        
      if( blink_counter == 400) lamp_state++;         // x msec-nel sotet/piros2
      if( blink_counter >= 800)                       // 2x msec-nel feher/piros1, szamlalo ujrainditas
      {
         lamp_state--;
         blink_counter= 0; 
      }
      _delay_ms( 1);
      blink_counter++;
   }
}
