/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.04.11.
*   Chip         -  ATtiny13A
*
*   Tavvezerelt jelzolampa modell - piros/sarga/zold
*
*                   +---------+
*                  ++ 1    8  |  +VCC
*   CONTROL - PB3  |  2    7  |  PB2 - L1
*   MIRROR  - PB4  |  3    6  |  PB1 - L2 (kozos, common)
*             GND  |  4    5  |  PB0 - L3
*                  +----------+
*
*   LED-ek vezerlese:  -  tukrozve:
*   piros: L1-1, L2-0  -  L3-1, L2-0
*   sarga: L3-1, L2-0  -  L1-1, L2-0
*   zold:  L1-0, L2-1  -  L3-0, L2-1
*
*******************************************************************************/

//                        000...000
#define F_CPU               1200000                   // 1,2 Mhz




#include "tkiraaly_attiny13a.h"
#include <util/delay.h>




// L1
#define L1_OUTPUT           PB2_OUTPUT
#define L1_0                PB2_0
#define L1_1                PB2_1

// L2
#define L2_OUTPUT           PB1_OUTPUT
#define L2_0                PB1_0
#define L2_1                PB1_1

// L3
#define L3_OUTPUT           PB0_OUTPUT
#define L3_0                PB0_0
#define L3_1                PB0_1

// CONTROL
#define CONTROL_PULLUP      PB3_PULLUP
#define CONTROL_R           PB3_R

// MIRROR
#define MIRROR_PULLUP       PB4_PULLUP
#define MIRROR_R            PB4_R

// State:
#define RED                 L1_1; L2_0; L3_0
#define RED_YELLOW          L1_1; L2_0; L3_1
#define GREEN               L1_0; L2_1; L3_1
#define YELLOW              L1_0; L2_0; L3_1
#define BLANK               L1_0; L2_0; L3_0

#define MIRRORED_RED        L3_1; L2_0; L1_0
#define MIRRORED_RED_YELLOW L3_1; L2_0; L1_1
#define MIRRORED_GREEN      L3_0; L2_1; L1_1
#define MIRRORED_YELLOW     L3_0; L2_0; L1_1




int main( void)
{
   U8 lamp_state= 0;                                  // lampa allapot szamlalo
   U8 pulse_lenght= 0;                                // impulzus hossz szamolasa
   U16 blink_counter= 0;                              // sarga villogas idozites szamlalo
   CONTROL_PULLUP;                                    // vezerles bemenete
   MIRROR_PULLUP;                                     // tukrozes jumper
   L1_OUTPUT;                                         // kimenetek
   L2_OUTPUT;
   L3_OUTPUT;
   for(;;)
   {
      if( MIRROR_R)                                   // lampa beallitasa MIRROR jumper szerint
      {
         switch( lamp_state )
         {

            case 0: RED; break;                       // piros - alap
            case 1: RED_YELLOW; break;                // piros + sarga
            case 2: GREEN; break;                     // zold
            case 3: YELLOW; break;                    // sarga
            case 4: YELLOW; break;                    // villogas sarga
            case 5: BLANK;                            // villogas fekete
         }
      }
      else
      {
         switch( lamp_state )
         {
            case 0: MIRRORED_RED; break;
            case 1: MIRRORED_RED_YELLOW; break;
            case 2: MIRRORED_GREEN; break;
            case 3: MIRRORED_YELLOW; break;
            case 4: MIRRORED_YELLOW; break;
            case 5: BLANK;
         }
      }     
      if( CONTROL_R == 0)                             // L - impulzus hossz szamolasa
      { 
         if( pulse_lenght < 9) pulse_lenght++;        // szamlalo ne fordulhasson korbe igen hosszu impulzus eseten
      }        
      else                                            // H - nincs impulzus
      {
         if( pulse_lenght)                            // volt impulzus, feldolgozas
         {
            if( pulse_lenght > 7)                     // 7 msec-nel hosszabb - alapba allitas
            {                      
               lamp_state= 0;
               blink_counter= 0;
            }
            else                                      // 7 msec-nel rovidebb - leptetes
            {
               if ( lamp_state < 4) lamp_state++;     // ne foroghasson korbe
            }
            pulse_lenght= 0;                          // kovetkezo impulzushoz
         }
      }        
      if( lamp_state > 3)                             // villogo sarga 
      {
         if( blink_counter == 300) lamp_state= 5;     // 300 msec-nel sotet
         if( blink_counter == 600)                    // 600 msec-nel sarga, szamlalo ujrainditas
         {
            lamp_state= 4;
            blink_counter= 0; 
         }
         blink_counter++;
      }
      _delay_ms( 1);
   }
}
