/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.04.11.
*   Chip         -  ATmega8
*
*   Tavvezerlo RC lamapahoz
*   
*   Egy-egy nyomogombbal a földre van kötve az ATmega8 PD4 és PD6 laba.
*   A PD4 (STEP) lenyomasara 5 msec, a PD6 (RESTART) lenymasara 10 msec hosszu
*   L impulzust ad ki az IC a PB2 (CONTROL) labon. 
*
*                              +---------+
*               NRESET - PC6  ++ 1    28 |  PC5 - ADC5 - SCL
*                  RXD - PD0  |  2    27 |  PC4 - ADC4 - SDA
*                  TXD - PD1  |  3    26 |  PC3 - ADC3
*                 INT0 - PD2  |  4    25 |  PC2 - ADC2
*                 INT1 - PD3  |  5    24 |  PC1 - ADC1
*             XCK - T0 - PD4  |  6    23 |  PC0 - ADC0
*                        VCC  |  7    22 |  GND
*                        GND  |  8    21 |  AREF
*         XTAL1 - OSC1 - PB6  |  9    20 |  AVCC
*         XTAL2 - OSC2 - PB7  | 10    19 |  PB5 - SCK
*                   T1 - PD5  | 11    18 |  PB4 - MISO
*                 AIN0 - PD6  | 12    17 |  PB3 - MOSI - OC2
*                 AIN1 - PD7  | 13    16 |  PB2 - NSS - OC1B
*                 ICP1 - PB0  | 14    15 |  PB1 - OC1A
*                             +----------+
*
*******************************************************************************/
#define F_CPU                4 MHZ




#include "tkiraaly_atmega8.h"
#include "tkiraaly_kbutton.cpp"
#include "tkiraaly_kled.cpp"
#include <util/delay.h>




int main( void)
{
   kbutton step( D_PORT_4);
   kbutton restart( D_PORT_6);       
   kled control( B_PORT_2);
   for(;;)
   {
      if( step.check())                // leptetes, 5 msec hosszu L impulzus
      {
         control.on();
         _delay_ms( 5);
         control.off();
      }
      if( restart.check())            // alapba allitas, 10 msec hosszu L impulzus
      {
         control.on();
         _delay_ms( 10);
         control.off();
      }
   }
}
